unit TelefonSpeedButton;

{
  Unit - TelefonSpeedButton, a unit like the Office - Style telephone - Button.
  Has to be connected to a DataSource.

  creation Date 7.6.98, by Stefan Diestelmann, SDS Software Wuppertal

  this component is freeware, can be used in Commercial or non commercial Applications.
  no Authors name must be shown.

  If you enhance it, please send me a copy sds.software@wtal.de

  thanks
}


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  RXCtrls, DB, DBCtrls, menus;

const
  DefaultTelefonarten: array[0..20] of string =
  (
     'Assistent',                  // 0
     'Geschftlich',               // 1
     'Geschftlich 2',             // 2
     'Fax geschftl.',             // 3
     'Rckmeldung',                // 4
     'Auto',                       // 5
     'Firma',                      // 6
     'Privat',                     // 7
     'Privat 2',                   // 8
     'Fax privat',                 // 9
     'ISDN',                       // 10
     'Mobiltelefon',               // 11
     'Weitere',                    // 12
     'Weiteres Fax',               // 13
     'Pager',                      // 14
     'Haupttelefon',               // 15
     'Funkruf',                    // 16
     'Telex',                      // 17
     'Texttelefon',                // 18
     'Praxis',                     // 19
     'Dienstlich'                  // 20
  );

type
  TTelefonSpeedButton = class(TRxSpeedButton)
  private
    { Private-Deklarationen }
    FDataLink: TFieldDataLink;
    procedure DataChange(Sender: TObject);
    procedure UpdateData(Sender: TObject);
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetFieldText: string;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
  protected
    { Protected-Deklarationen }
    pmTelefonMenue : TPopupMenu;
    miTelefonarten : array [0..20] of TMenuItem;
    procedure CreateMenu;
    procedure SelectTelefonart(Sender: TObject);
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published-Deklarationen }
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;

procedure Register;

implementation

//******************* TTelefonSpeedButton.Create *************************

constructor TTelefonSpeedButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnUpdateData := UpdateData;
  CreateMenu;
  DropDownMenu := pmTelefonMenue;
end;

destructor TTelefonSpeedButton.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;

procedure TTelefonSpeedButton.SelectTelefonart(Sender: TObject);
begin
   FDataLink.Edit;
   Caption := TMenuItem(Sender).Caption;
   FDataLink.Field.Text := Caption;
   FDataLink.Modified;
end;

procedure TTelefonSpeedButton.CreateMenu;
var
  i : integer;
begin
  try
    pmTelefonMenue := TPopupMenu.Create(Self);
    for i := 0 to 20 do begin
      miTelefonarten[i] := TMenuItem.Create(Self);
      miTelefonarten[i].Caption := DefaultTelefonarten[i];
      miTelefonarten[i].OnClick := SelectTelefonart;
      pmTelefonMenue.Items.Add(miTelefonarten[i]);
    end;
    pmTelefonMenue.Name := 'pmTelefonMenueIntern';
  except
    pmTelefonMenue.Free;
  end;
end;

//******************* TTelefonSpeedButton: Datenbankverknpfung *************************

procedure TTelefonSpeedButton.Loaded;
begin
  inherited Loaded;
  if (csDesigning in ComponentState) then DataChange(Self);
end;

procedure TTelefonSpeedButton.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

function TTelefonSpeedButton.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TTelefonSpeedButton.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

function TTelefonSpeedButton.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

procedure TTelefonSpeedButton.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

function TTelefonSpeedButton.GetFieldText: string;
begin
  if FDataLink.Field <> nil then
    Result := FDataLink.Field.DisplayText
  else
    if csDesigning in ComponentState then Result := Name else Result := '';
end;

procedure TTelefonSpeedButton.DataChange(Sender: TObject);
begin
  Caption := GetFieldText;
end;

procedure TTelefonSpeedButton.UpdateData(Sender: TObject);
begin
  FDataLink.Field.Text := Caption;
end;

procedure TTelefonSpeedButton.CMGetDataLink(var Message: TMessage);
begin
  Message.Result := Integer(FDataLink);
end;

procedure TTelefonSpeedButton.CMExit(var Message: TCMExit);
begin
  try
    FDataLink.UpdateRecord;
  except
    raise;
  end;
end;

procedure Register;
begin
  RegisterComponents('SDS', [TTelefonSpeedButton]);
end;

end.
